package pps.core.prediction.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.text.CharSequenceUtil;
import com.alibaba.excel.EasyExcelFactory;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.excel.util.EasyExcelUtil;
import pps.core.common.excel.util.ExcelStyleTool;
import pps.core.prediction.entity.PlantPowerDataEnt;
import pps.core.prediction.mapper.PlantPowerDataMapper;
import pps.core.prediction.service.data.plant_power_data.ExcelPlantPowerTemplate;
import pps.core.prediction.service.data.plant_power_data.ImportFileInput;
import pps.core.prediction.service.data.plant_power_data.QueryPlantPowerDataInput;
import pps.core.prediction.service.data.plant_power_data.QueryPlantPowerDataOutput;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.tool.XStorageTool;
import xstartup.data.XFileResult;
import xstartup.data.XFileType;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiUpload;
import xstartup.feature.mybatis.helper.XMapperHelper;

import java.math.BigDecimal;

@XService
@XText("电站实际发电数据")
public class PlantPowerDataService {

    @XApiGet
    public XPageResult<QueryPlantPowerDataOutput> queryPlantPowerDataList(XContext context, QueryPlantPowerDataInput input) {
        PlantPowerDataMapper mapper = context.getBean(PlantPowerDataMapper.class);
        QueryWrapper<PlantPowerDataEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(PlantPowerDataEnt::getPlantId, input.getPlantId());
        if (CharSequenceUtil.isNotBlank(input.getStartTime()))
            queryWrapper.lambda().ge(PlantPowerDataEnt::getDataDate, input.getStartTime());
        if (CharSequenceUtil.isNotBlank(input.getEndTime()))
            queryWrapper.lambda().ge(PlantPowerDataEnt::getDataDate, input.getEndTime());
        queryWrapper.lambda().orderByDesc(PlantPowerDataEnt::getDataDate);
        return XMapperHelper.query(mapper, input, queryWrapper, QueryPlantPowerDataOutput.class);
    }

    /**
     * 下载导入模板
     *
     * @param context 上下文
     * @return {@link XFileResult }
     */
    @XText("下载电站实际发电数据导入模板")
    @XApiGet
    public XFileResult downloadImportTemplate(XContext context) {
        DateTime date = DateUtil.date();
        String fileSavePath = XStorageTool.getAbsolutePath("/temp/excel/导入模板_" + DateUtil.formatDate(date) + ".xlsx");
        //生成模板
        EasyExcelFactory.write(FileUtil.touch(fileSavePath))
                .registerWriteHandler(ExcelStyleTool.getStyleStrategy())
                .autoCloseStream(Boolean.TRUE)
                .build()
                .write(CollUtil.list(false, ExcelPlantPowerTemplate.builder()
                        .dataDate(date)
                        .windSpeed(BusinessConstant.BIG_DECIMAL_6_67)
                        .windDirection(BigDecimal.ZERO)
                        .temperature(BusinessConstant.BIG_DECIMAL_4)
                        .pressure(BigDecimal.ONE)
                        .humidity(BusinessConstant.BIG_DECIMAL_3)
                        .planeIrradiance(BusinessConstant.BIG_DECIMAL_2)
                        .horizontalIrradiance(BusinessConstant.BIG_DECIMAL_4)
                        .power(BusinessConstant.BIG_DECIMAL_60)
                        .build()), EasyExcelUtil.writeSelectedSheet(ExcelPlantPowerTemplate.class, 0, "导入模板"))
                .finish();
        return XFileResult.success(XFileType.APPLICATION_XLSX, fileSavePath);
    }

    /**
     * 导入工厂电力数据
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult }
     */
    @XText("导入电站实际发电数据")
    @XApiUpload
    public XServiceResult importPlantPowerData(XContext context, ImportFileInput input) {
        return XServiceResult.OK;
    }
}